#[non_exhaustive]pub enum AcceptError {
Connecting {
source: ConnectingError,
meta: Meta,
},
Connection {
source: ConnectionError,
meta: Meta,
},
MissingRemoteEndpointId {
source: RemoteEndpointIdError,
meta: Meta,
},
NotAllowed {
meta: Meta,
},
User {
source: AnyError,
meta: Meta,
},
}Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Implementations
Sourceimpl AcceptError
impl AcceptError
Sourcepub fn from_err<T>(value: T) -> AcceptError
pub fn from_err<T>(value: T) -> AcceptError
Creates a new user error from an arbitrary error type.
Sourcepub fn from_boxed(value: Box<dyn Error + Sync + Send>) -> AcceptError
pub fn from_boxed(value: Box<dyn Error + Sync + Send>) -> AcceptError
Creates a new user error from an arbitrary boxed error.