pub enum GossipEvent {
NeighborUp(PublicKey),
NeighborDown(PublicKey),
Received(Message),
Lagged,
}Expand description
Events emitted from a gossip topic.
These are the events emitted from a GossipReceiver.
Variants
NeighborUp(PublicKey)
We have a new, direct neighbor in the swarm membership layer for this topic.
NeighborDown(PublicKey)
We dropped direct neighbor in the swarm membership layer for this topic.
Received(Message)
We received a gossip message for this topic.
Lagged
We missed some messages because our GossipReceiver was not progressing fast enough.