Trait interfaces for pluggable DID and IPFS publishing backends.
Implement these traits to decouple domain logic from a specific transport or storage backend.
Trait interfaces for pluggable DID and IPFS publishing backends.
Implement these traits to decouple domain logic from a specific transport or storage backend.