pub struct Empty;Expand description
An empty preset that doesn’t set anything on the builder.
This doesn’t set mandatory builder options, so using this in
Endpoint::bind(presets::Empty) will always fail.
However, it can be useful, if you want control over all mandatory options
yourself, by using Endpoint::builder(presets::Empty).
If you prefer a minimal version that is guaranteed to work, see the
Minimal preset.