pub struct SecretKey(/* private fields */);Expand description
A secret key.
Implementations
Sourceimpl SecretKey
impl SecretKey
Sourcepub fn generate() -> SecretKey
pub fn generate() -> SecretKey
Generate a new SecretKey with a randomness generator.
This uses the default random number generator from the rand crate.
If you want to customize how the randomness is generated, use
Self::from_bytes instead and generate the 32 bytes yourself:
// Create a random number generator.
let mut rng = rand::rng();
// Use it to generate the 32 bytes that make up a secret key.
let secret_key = SecretKey::from_bytes(&rng.random());Sourcepub fn sign(&self, msg: &[u8]) -> Signature
pub fn sign(&self, msg: &[u8]) -> Signature
Sign the given message and return a digital signature
Sourcepub fn to_bytes(&self) -> [u8; 32]
pub fn to_bytes(&self) -> [u8; 32]
Convert this to the bytes representing the secret part. The public part can always be recovered.
Sourcepub fn from_bytes(bytes: &[u8; 32]) -> SecretKey
pub fn from_bytes(bytes: &[u8; 32]) -> SecretKey
Create a secret key from its byte representation.