pub struct N0;Expand description
Configures the endpoint to use the n0 defaults
Currently this consists of
- the DNS Address Lookup service.
- the default relay servers provided by Number 0.
- setting the
rustls::crypto::CryptoProviderto ring or aws-lc-rs, depending on which feature is enabled in iroh (preferring ring if both are enabled).
Due to the last point, this preset is only available with the ring or
aws-lc-rs preset installed.
If you want to set your own crypto provider, we recommend copying the
implementation of this preset into your own and setting the appropriate crypto
provider there.
The default address lookup service publishes to and resolves from the
n0.computer dns server iroh.link.
This is equivalent to adding both a crate::address_lookup::PkarrPublisher
and a crate::address_lookup::DnsAddressLookup, both configured to use the
n0.computer dns server.
This will by default use N0_DNS_PKARR_RELAY_PROD.
When in tests, or when the test-utils feature is enabled, this will use the
N0_DNS_PKARR_RELAY_STAGING.