N0DisableRelay

Struct N0DisableRelay 

pub struct N0DisableRelay;
Configures the endpoint to use the n0 defaults, but with relay mode disabled.

Currently this consists of

  • setting RelayMode::Disabled
  • the DNS Address Lookup service, that publishes IP addresses rather than relay urls.

The default address lookup service publishes to and resolves from the n0.computer dns server iroh.link.

This is equivalent to adding both a crate::address_lookup::PkarrPublisher and a crate::address_lookup::DnsAddressLookup, both configured to use the n0.computer dns server.

This will by default use N0_DNS_PKARR_RELAY_PROD. When in tests, or when the test-utils feature is enabled, this will use the N0_DNS_PKARR_RELAY_STAGING.